<?php

/*
 * iPay88 Gateway for WooCommerce
 *
 * Description: Allows you to use iPay88 e-Wallet Payment with the WooCommerce plugin.
 * Plugin URI: https://www.ipay88.co.id/
 * Author: iPay88 Indonesia
 * License: The MIT License (MIT)
 *
 */
 
class ipay88_qr_gateway extends ipay88_QR_Abstract 
{
	//editor: Alda Mustafri
	
    public $id 			= 'ipay88_qr';
    public $has_fields 	= true;
	
    protected $ipay88_payment_method_selected;

    public function __construct() 
	{
        parent::__construct();
		
        $this->icon 				= IPAY88_QR_PLUGIN_URL . '/assets/images/qris.svg';
        $this->method_title 		= __('iPay88 QR Code Payment', IPAY88_QR_TEXT_DOMAIN);
        $this->method_description 	= __('Allows you to use iPay88 QR Code Payment with the WooCommerce plugin.', IPAY88_QR_TEXT_DOMAIN);
       
	    $this->title = isset($this->settings['title']) ? $this->settings['title'] : "iPay88 QR Code Payment.";
        $this->notify_url = add_query_arg('wc-api', 'ipay88_qr_gateway', home_url('/'));
		
        // Our Actions
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));
        add_action('woocommerce_api_' . $this->id . '_gateway', array($this, 'ipn_response'));
    	
		$this->image_ext 	= 'png';
		$this->hash_amount 	= 0;
		$this->formatted_amount = 0;
		

		// Define user set variables
		$this->title					= $this->settings['title'];
		$this->description				= $this->settings['description'];
		$this->enabled					= isset( $this->settings['enabled'] ) ? $this->settings['enabled'] : 'no';
		$this->MerchantCode				= $this->settings['merchant_code'];
		$this->MerchantKey				= $this->settings['merchant_key'];
		$this->PaymentId				= $this->settings['ipay88_qr_config'];
	}

    //Initial ipay88 Configs
    function init_configs($isPingback = false) 
	{
        ipay88_QR_Config::getInstance()->set(array( 'merchant_code' => $this->settings['merchant_code'],
            									 'merchant_key' => $this->settings['merchant_key']));
    }

    function getConfigOptionByKey($key) 
	{
        return $this->settings[$key];
    }

    //@param $order
    function receipt_page($order) 
	{
        $this->init_configs();
        echo $redirect_form = $this->generate_ipay88_form($order);

		echo '<p>'. __( 'Thank you for your order, please click the button below to pay with iPay88.', IPAY88_QR_TEXT_DOMAIN ) .'</p>';

		echo $redirect_form = $this->generate_ipay88_form( $order );
    }

    function dd($data) 
	{
        echo "<pre>";
        print_r($data);
        echo "</pre>";
    }

	/**
	 * Generate the sha1 control signature. <br/>
	 * Used in both the request and the response to validate the authenticity of the message.
	 *
	 * @global object $woocommerce
	 * @param array $params The request or response parameters
	 * @param bool $is_response Are the parameters from the response message
	 * @return string The sha1 generated generate_ipay88_form
	 **/
	private function generate_sha1_signature( $params, $is_response = true )
	{
		$string = '';
		if ( $is_response ) {
			$this->format_amount( str_replace( ',', '', $params['Amount'] ) );
			$string = $this->MerchantKey . $this->MerchantCode . $params['PaymentId'] . $params['RefNo'] . $this->hash_amount/100 . $params['Currency'] . $params['Status'];
		} else {
			$string = $this->MerchantKey . $this->MerchantCode . $params['RefNo'] . $this->hash_amount . $params['Currency'];
		}

		return base64_encode( $this->hex2bin( sha1( $string ) ) );
	}

	function hex2bin( $hexSource )
	{
		$bin = '';
		for ( $i = 0; $i < strlen( $hexSource ); $i = $i + 2 ) {
			$bin .= chr( hexdec( substr( $hexSource, $i, 2 ) ) );
		}

		return $bin;
	}
	
	/**
	 * Generate iPay88 form
	 **/
    function generate_ipay88_form($order_id) 
	{
		$order = new WC_Order( $order_id );

		$desc = '';
		
		if ( 0 < sizeof( $order->get_items() ) ) 
		{
			foreach ( $order->get_items() as $item ) 
			{
				if ( $item['qty'] ) 
				{
					$item_name = preg_replace("/<br\s*\/?>/", "", $item['name']);
					$item_name = htmlspecialchars($item_name);
					// $item_meta = WC_Compat_iPay88_QR::get_order_item_meta( $item );

					// if ($meta = $item_meta->display( true, true )) 
					// {
					// 		$item_name .= ' ('.$meta.')';
					// }

					$desc .= $item['qty'] .' x '. $item_name . ', ';
				}
			}
			
			//Add the description
			$desc = substr($desc, 0, -2 );
		}

		
		$currency = get_woocommerce_currency();

		$responseUrl 	=
		WC_Compat_iPay88_QR::force_https(add_query_arg('wc-api', 'ipay88_qr_gateway', home_url('/')));

		$backendUrl 	=
		WC_Compat_iPay88_QR::force_https(add_query_arg('iPay88_response', 'backend', add_query_arg('wc-api', 'ipay88_qr_gateway', home_url('/'))));

		// Format the order total
		$this->format_amount( $order->get_total() );

		$refNo			= date_format($order->get_date_created(), "Ymd") . "-" . $order_id;
		$paymentId 		= $this->PaymentId;

		$ipay88_args = array(
			'MerchantCode'	=> $this->MerchantCode,
			'RefNo'			=> $refNo,
			'Amount'		=> $this->formatted_amount,
			'Currency'		=> $currency,
			'ProdDesc'		=> $desc,
			'UserName'      => $order->billing_first_name .' '. $order->billing_last_name,
			'UserEmail'		=> $order->billing_email,
			'UserContact'	=> $order->billing_phone,
			'ResponseURL'	=> $responseUrl,
			'BackendURL'	=> $backendUrl,
			'PaymentId'		=> $paymentId
		);

       
		//Add signature
		$ipay88_args['signature'] = $this->generate_sha1_signature( $ipay88_args, false );

		$isTest 		= $this->getConfigOptionByKey('test_mode');
		if($isTest == 'no')
		{
			$getwayUrl = ipay88_QR_Indonesia::URL_PAYMENT_PRODUCTION;
		}
		
		else
		{
			$getwayUrl = ipay88_QR_Indonesia::URL_PAYMENT_SANDBOX;
		}

		$wiget = new ipay88_QR_Wiget($ipay88_args);
        $wiget->setAction($getwayUrl);

        return $wiget->generateRedirectForm();
    }

    // Process the order after payment is made
    // @param int $order_id
    // @return array
    function process_payment($order_id) 
	{
		$order = wc_get_order($order_id);
		
		$payment_method_selected 			= $_POST['ipay88_bank_payment_method'];
        $_SESSION['posted_payment_type']	= $_POST['ipay88_bank_payment_method'];

		$orderkey = !method_exists($order, 'get_order_key') ? $order->order_key : $order->get_order_key();
		$orderid = !method_exists($order, 'get_id') ? $order->id : $order->get_id();

        return array(	'result' 	=> 'success',
            			'redirect' 	=> add_query_arg('key', $orderkey, add_query_arg(array(
                        'order-pay' => $orderid,
                        'ipay88_payment_method_selected' => $payment_method_selected),
                    	$order->get_checkout_payment_url(true))));
    }

    // Check the response from ipay88's Servers
    function ipn_response() 
	{
		WC_iPay88_QR::add_debug_log("ipn_response " . print_r($_REQUEST, true));
        global $woocommerce;

        $refNo = $_POST['RefNo'];
		$splitReffNo = explode("-", $refNo);
		$ReffNo_Date	= $splitReffNo[0];
		$ReffNo_Number	= $splitReffNo[1];

		$order = wc_get_order($ReffNo_Number);

        if (!$order) 
		{
            die('The order is Invalid!');
        }

        $merchantKey = $this->getConfigOptionByKey('merchant_key');
        $transaction = new ipay88_QR_Transaction($_POST);

        $transaction->setMerchantKey($merchantKey);
		$transaction->setSignature($_POST['Signature']);
        
		WC_iPay88_QR::add_debug_log("transaction " . print_r($transaction, true));

        if($transaction->isPaid()) 
		{
			if($order->get_status() == 'pending' || $order->get_status() == 'on-hold')
			{
				WC_iPay88_QR::add_debug_log("transaction isPaid " . print_r($transaction, true));
				$order->add_order_note(__('The Order has been Paid by iPay88 Payment Gateway with QR. The customer has paid IDR ' . ($_POST['Amount'] / 100) . ' - Transaction Id: ' . $_POST['TransId'], IPAY88_QR_TEXT_DOMAIN));
			
				$order->payment_complete();
				$order->update_status('processing');
				$order->save();
			}

			if($order->get_status() == 'completed')
			{
				$woocommerce->cart->empty_cart();
			}

            if (isset($_REQUEST['iPay88_response'])) {
				die('RECEIVEOK');
			} else {
				status_header(200);
				wc_add_notice('Your Payment success.', 'success');
				wp_redirect($this->get_return_url($order));

				// Return thank you page redirect
				//return array('result'   => 'success', 'redirect' => $this->get_return_url($order));
			}
        } 
		
		elseif ($transaction->isWaitingPayment()) 
		{
			WC_iPay88_QR::add_debug_log("transaction isWaitingPayment " . print_r($transaction, true));
            $order->add_order_note(__('The Order is waiting payment via ipay88 Payment Gateway - Transaction Id: ' . $_POST['TransId'], IPAY88_QR_TEXT_DOMAIN));
			
            $order->update_status('on-hold');
			$order->save();

			if (isset($_REQUEST['iPay88_response'])) {
				die('PENDING');
			} else {
				wp_redirect($this->get_return_url($order));
			}
        } 
		
		elseif ($transaction->isPaymentinProgress()) 
		{
			WC_iPay88_QR::add_debug_log("transaction isPaymentinProgress " . print_r($transaction, true));
            $order->add_order_note(__('The Order is waiting payment via ipay88 Payment Gateway - Transaction Id: ' . $_POST['TransId'], IPAY88_QR_TEXT_DOMAIN));
			
            $order->update_status('on-hold');
			$order->save();

			if (isset($_REQUEST['iPay88_response'])) {
				die('PENDING');
			} else {
				wp_redirect($this->get_return_url($order));
			}
        } 
		
		
		else 
		{
			WC_iPay88_QR::add_debug_log("transaction ErrDesc " . print_r($transaction, true));
			global $woocommerce;
			wc_add_notice('Your Payment Failed. Please try again or use another payment option.', 'error');

			$order->cancel_order(__('Reason: ' . $_POST['ErrDesc'], IPAY88_QR_TEXT_DOMAIN));

			if (isset($_REQUEST['iPay88_response'])) {
				die('BACKENDFAILED');
			} else {
				return wp_redirect(WC()->cart->get_checkout_url());
			}
        }
    }
	
	// Format the two amounts we need.
	// One for hashing
	// One for request parameter
	// @param double $amount
	function format_amount($amount) 
	{
		if (is_numeric($amount)) 
		{
			$this->hash_amount 		= number_format($amount, 2, '', '');
			$this->formatted_amount = number_format($amount, 2, '', '');
		}
	}
}
